<?php
// $Id: regcode.admin.php,v 1.1.2.9 2009/07/22 20:44:36 aidan Exp $

require_once('regcode.api.php');

/**
 * Return the form associated with the module settings.
 *
 * @return
 *   The settings form.
 */
function regcode_admin_settings() {
  $form = array();
  
  $form['regcode_optional'] = array(
    '#type' => 'checkbox',
    '#title' => t("Make registration code optional"),
    '#default_value' => variable_get('regcode_optional', 0),
    '#description' => t('If checked, users can register without a registration code. ') .
                      t('Alternatively, there is also a permission to allow certain roles (e.g. admins) to bypass´registration code entry for administratively added users.'),
  );
  
  $form['regcode_fieldset_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field set title"),
    '#description' => t('The title of the registration code fieldset'),
    '#default_value' => variable_get('regcode_fieldset_title', 'Registration Code'),
  );

  $form['regcode_field_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field label"),
    '#description' => t('The label of the registration code textfield'),
    '#default_value' => variable_get('regcode_field_title', 'Registration Code'),
  );

  $form['regcode_field_description'] = array(
    '#type' => 'textfield',
    '#title' => t("Field description"),
    '#description' => t('The description under the registration code textfield'),
    '#default_value' => variable_get('regcode_field_description', 'Please enter your registration code.'),
  );

  return system_settings_form($form);
}


/**
 * Create a registration code
 *
 * @return
 *   The settings form.
 */
function regcode_admin_create() {
  $form = array();

  // Basics
  $form['regcode_create'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Basics"),
    '#description'   => t('Basic options for the new registration code'),
  );
  
  $form['regcode_create']['regcode_create_category'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Category"),
    '#size'          => 30,
    '#autocomplete_path' => 'admin/user/regcodes/autocomplete',
    '#description'   => t('Assign a category for administration purposes, to aid code management.'),
  );  
  
  $form['regcode_create']['regcode_create_code'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Registration code"),
    '#description'   => t('Leave blank to have code generated. Used as prefix when <em>Number of codes</em> is greater than 1.'),
  );

  $form['regcode_create']['regcode_create_maxuses'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Maximum uses"),
    '#default_value' => 1,
    '#size'          => 10,
    '#description'   => t('How many times this code can be used to register (leave blank for unlimited).'),
  );

  $form['regcode_create']['regcode_create_begins'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Active from"),
    '#description'   => t('When this code should activate (leave blank to activate immediately). Accepts any date format that strtotime can handle.'),
    '#size'          => 30,
  );
  
  $form['regcode_create']['regcode_create_expires'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Expires on"),
    '#description'   => t('When this code should expire (leave blank for no expiry). Accepts any date format that strtotime can handle.'),
    '#size'          => 30,
  );

  // Bulk
  $form['regcode_create_bulk'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Code generation"),
    '#description'   => t('Settings used when generating codes'),
  );

  $form['regcode_create_bulk']['regcode_create_number'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Number of codes to generate"),
    '#size'          => 10,
    '#default_value' => 1,
    '#description'   => t('The number of codes to generate'),
  );
  
  $form['regcode_create_bulk']['regcode_create_length'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Code size"),
    '#size'          => 10,
    '#default_value' => 10,
    '#description'   => t('The length of the codes created'),
  );
  
  $form['regcode_create_bulk']['regcode_create_format'] = array(
    '#type'          => 'select',
    '#default_value' => variable_get('regcode_generate_format', 'alpha'),
    '#title'         => t('Format of the generated codes'),
    '#options'       => array('alpha' => t('Letters'), 'numeric' => t('Numbers'), 'alphanum' => t('Numbers & Letters'), 'hexadec' => t('Hexadecimal')),
  );
  
  $form['regcode_create_bulk']['regcode_create_case'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Uppercase generated codes'),
    '#default_value' => variable_get('regcode_generate_case', ''),
  );

  $form['regcode_create_bulk_submit'] = array(
    '#type'          => 'submit',
    '#value'         => t("Create codes"),
  );
  
  return $form;
}


/**
 * Validator for create form
 */
function regcode_admin_create_validate($form, &$form_state) {
  if (!empty($form_state['values']['regcode_create_begins']) &&
      strtotime($form_state['values']['regcode_create_begins']) < 1) {

    form_set_error('regcode_create_begins', 'Invalid begin date');      
  }
  
  if (!empty($form_state['values']['regcode_create_expires']) &&
      strtotime($form_state['values']['regcode_create_expires']) < 1) {

    form_set_error('regcode_create_expires', 'Invalid expiry date');      
  }

}


/**
 * Submit handler for regcode_admin_create
 */
function regcode_admin_create_submit($form, &$form_state) {
  // Build our new code
  $code = array();
  if (!empty($form_state['values']['regcode_create_bulk_begins'])) {
    $code['begins'] = strtotime($form_state['values']['regcode_create_begins']);
  }
  if (!empty($form_state['values']['regcode_create_expires'])) {
    $code['expires'] = strtotime($form_state['values']['regcode_create_expires']);
  }
  $code['is_active'] = 1;
  $code['category']  = $form_state['values']['regcode_create_category'];
  $code['maxuses']   = $form_state['values']['regcode_create_maxuses'];
  
  for ($i = 0; $i < (int) $form_state['values']['regcode_create_number']; $i++) {
    $code['code'] = $form_state['values']['regcode_create_code'];

    // Generate a code
    if (empty($code['code']) || $form_state['values']['regcode_create_number'] > 1) {
      $gen = regcode_generate_code($form_state['values']['regcode_create_length'],
        $form_state['values']['regcode_create_format'], 
        $form_state['values']['regcode_create_case']);
      $code['code'] .= $gen;
    }
    
    // Save code
    if (regcode_save_code($code, 'skip')) {
      drupal_set_message(t('Created registration code (@code)', array('@code' => $code['code'])));
    } else {
      drupal_set_message(t('Unable to create code (@code) as code already exists', array('@code' => $code['code'])), 'warning');  
    }
  }
}


/**
 * Return the code list page content with(in) the according filter form
 *
 * @return
 *   The settings form.
 */
function regcode_admin_list($form_state) {
  // Filters
  $filters['is_active'] = array('1' => t('Yes'), '0' => t('No'));
  $filters['category']  = drupal_map_assoc(regcode_categories());

  
  // Form
  $form = array();
  
  $form['filter'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter results:'),
  );
  
  $form['filter']['category'] = array(
    '#type' => 'select',
    '#title' => t('Category'),
    '#options' => array('__all__' => t('-- All --')) + $filters['category'],
    '#default_value' => variable_get('regcode_filter_category', '__all__'),
    '#prefix' => '<div class="container-inline">',
  );
  
  $form['filter']['is_active'] = array(
    '#type' => 'select',
    '#title' => t('Enabled'),
    '#default_value' => variable_get('regcode_filter_is_active', '__all__'),
    '#options' => array('__all__' => t('-- All --')) + $filters['is_active'],
  );
  
  $form['filter']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter list'),
    '#submit' => array('regcode_admin_list_filter'),
  );
  
  $form['filter']['export'] = array(
    '#type' => 'submit',
    '#value' => t('Export to CSV'),
    '#submit' => array('regcode_admin_list_export'),
    '#suffix' => '</div>',
  );


  // Generate the tablesort
  require_once 'regcode.install';
  $schema = regcode_schema('regcode');
  $header = array();
  foreach ($schema['regcode']['fields'] as $field => $value) {
    $header[] = array('data' => $value['#heading'], 'field' => $field);
  }
  $sort = tablesort_sql($header);
    
  // Generate the WHERE
  $active = array();
  $where = '';
  $category = variable_get('regcode_filter_category', '__all__');
  if ($category !== '__all__' && isset($filters['category'][$category])) {
    $active[] = sprintf('category = "%s"', $category);
  }
  $is_active = variable_get('regcode_filter_is_active', '__all__');
  if ($is_active !== '__all__' && isset($filters['is_active'][$is_active])) {
    $active[] = sprintf('is_active = %d', $is_active);
  }
  if (count($active)) {
    $where = ' WHERE ' . implode($active, ' AND ');
  }
  
  // Generate the SQL
  $keys       = array_keys($schema['regcode']['fields']);
  $sql        = 'SELECT ' . implode(',', $keys) . ' FROM {regcode}' . $where . $sort;
  $result     = pager_query($sql, 50);
  $codes      = array();
  while ($row = db_fetch_array($result)) {
    $codes[] = _regcode_admin_prepare_display($row);
  }
  
  $visible = regcode_count($where);
  $total   = regcode_count();
  $form['filter']['recordcount'] = array(
    '#type' => 'markup',
    '#value' => '<div>' . t('Paging @visible records out of @total.', array('@visible' => $visible, '@total' => $total)) . '</div>',
  ); 
  
  // Output
  if (empty($codes)) {
    $output = '<p>' . t('&raquo; No codes found.') . '</p>';
  } else {
    $output .= theme('table', $header, $codes);
    $output .= theme('pager');
  }
  
  $form['list'] = array(
      '#type'  => 'markup',
      '#value' => $output,
  );

  return $form;
}


/**
 *
 */
function regcode_admin_list_filter($form, &$form_state) {  
  $filters = array('category', 'is_active');
  
  foreach ($filters as $filter) {
    if (isset($form_state['values'][$filter])) {
      variable_set('regcode_filter_' . $filter, $form_state['values'][$filter]);
    }  
  }
}


/**
 *
 */
function regcode_admin_list_export($form, &$form_state) {
  require_once 'regcode.install';
  $schema = regcode_schema('regcode');

  $filters = array('category', 'is_active');
  
  foreach ($filters as $filter) {
    if (isset($form_state['values'][$filter])) {
      variable_set('regcode_filter_' . $filter, $form_state['values'][$filter]);
    }  
  }
  $filters['is_active'] = array('1' => 'Yes', '0' => 'No');
  $filters['category']  = drupal_map_assoc(regcode_categories());

  
  $active = array();
  $where = '';
  $category = variable_get('regcode_filter_category', '__all__');
  if ($category !== '__all__' && isset($filters['category'][$category])) {
    $active[] = sprintf('category = "%s"', $category);
  }
  $is_active = variable_get('regcode_filter_is_active', '__all__');
  if ($is_active !== '__all__' && isset($filters['is_active'][$is_active])) {
    $active[] = sprintf('is_active = %d', $is_active);
  }
  if (count($active)) {
    $where = ' WHERE ' . implode($active, ' AND ');
  }
  
  $filename = file_create_filename('regcode.csv', file_directory_temp());
  
  // Generate the SQL
  $keys       = array_keys($schema['regcode']['fields']);
  $sql        = 'SELECT ' . implode(',', $keys) . ' FROM {regcode}' . $where;
  $result     = db_query($sql);
  $codes      = array();
  
  $fh = fopen($filename, 'w+');
  fputcsv($fh, $keys);
  $i = 0;
  while ($row = db_fetch_array($result)) {
    $i++;
    fputcsv($fh, $row);
  }
  fclose($fh);
  $headers = array(
    'Content-Type: text/csv',
    'Content-Disposition: attachment; filename="regcode.csv"',
    );
  file_transfer($filename, $headers);
  
  // Export
  drupal_set_message(t('@count records exported.', array('@count' => $i)));
}


/**
 * Return the form associated with the registration code import admin page
 *
 * @return
 *   The import form array
 */
function regcode_admin_import() {
  require_once 'regcode.install';

  // Build the descriptions
  $action_text  = '<p>' . t('Action to perform when importing the data.') . '</p>';
  $action_text .= '<ul><li>' . t('<strong>Skip</strong>: Add new codes skipping those which already exists.') . '</li>';
  $action_text .= '<li>' . t('<strong>Overwrite</strong>: Add new codes overwriting those which already exist.') . '</li>';
  $action_text .= '<li>' . t('<strong>Clean</strong>: Erases *all* existing codes before importing new codes.') . '</li></ul>';

  $fieldorder_text  = '<p>' . t('Comma separated list mapping regcode fields to CSV fields, e.g. "IGNORE, code, rid, info".') . '</p>';
  $fieldorder_text .= '<p>' . t('If you leave this field blank, the titles from the first row will be used.') . '</p>';
  $fieldorder_text .= '<ul>';
  $schema = regcode_schema('regcode');
  foreach ($schema['regcode']['fields'] as $field => $value) {
    if ($value['#exposed'] === TRUE) {
      $fieldorder_text .= sprintf('<li><strong>%s</strong>: %s</li>', $field, $value['description']);
    }
  }
  $fieldorder_text .= '</ul>';
                             
  // Build the form                         
  $form['#attributes']['enctype'] = 'multipart/form-data';

  $form['regcode_import_file'] = array(
    '#type' => 'file',
    '#title' => t("File"),
    '#description' => t('File upload with data to add/import (plaintext list or CSV format)'),
  );

  $form['regcode_import_action'] = array(
    '#type' => 'select',
    '#title' => t("Action"),
    '#default_value' => variable_get('regcode_import_action', 'skip'),
    '#options' => array('skip' => t('Skip'), 'overwrite' => t('Overwrite'), 'clean' => t('Clean')),
    '#description' => $action_text,
  );

  $form['regcode_import_delimiter'] = array(
    '#type'          => 'textfield',
    '#size'          => 5,
    '#title'         => t("Field delimiter"),
    '#description'   => t('Character used as delimiter in csv import of registration codes'),
    '#default_value' => variable_get('regcode_import_delimiter', ','),
  );

  $form['regcode_import_enclosure'] = array(
    '#type'          => 'textfield',
    '#size'          => 5,
    '#title'         => t("Text enclosure"),
    '#default_value' => variable_get('regcode_import_enclosure', '"'),
    '#description'   => t('Character used as enclosure around text content fields on CSV import.'),
  );
  
  $form['regcode_import_fieldorder'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Field order"),
    '#default_value' => variable_get('regcode_import_fieldorder', 'code'),
    '#description'   => $fieldorder_text,
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}


/**
 * Handle the processing of a submitted import form
 *
 */
function regcode_admin_import_validate($form, &$form_state) {
  variable_set('regcode_import_delimiter',  $form_state['values']['regcode_import_delimiter']);
  variable_set('regcode_import_enclosure',  $form_state['values']['regcode_import_enclosure']);
  variable_set('regcode_import_fieldorder', $form_state['values']['regcode_import_fieldorder']);
  
  // Validators for file
  $validators = array(
    'file_validate_extensions' => array('csv', 'txt'),
  );

  // Save the file
  if ($file = file_save_upload('regcode_import_file', $validators, file_directory_path())) {
    $form_state['values']['fileobj'] = $file;
    drupal_set_message('File successfully saved');
  } else {
    form_set_error('regcode_import_file', 'Unable to parse CSV file');
    return false;
  }
}


/**
 * Handle the processing of a submitted import form
 *
 */
function regcode_admin_import_submit($form, &$form_state) {
  
  // Set some variblaes
  $action    = $form_state['values']['regcode_import_action'];
  $delimiter = $form_state['values']['regcode_import_delimiter'];
  $enclosure = $form_state['values']['regcode_import_enclosure'];
  $fields    = explode(',', $form_state['values']['regcode_import_fieldorder']);
  $fields    = array_map('trim', $fields);
  $rows = $success = 0;
  
  // Parse and save
  $fh        = fopen($form_state['values']['fileobj']->destination, 'r');
  while ($row = fgetcsv($fh, 1000, $delimiter, $enclosure)) {
    $rows++;
    $row  = array_slice($row, 0, count($fields));
    $code = array_combine($fields, $row);
    
    // Check dates
    if (!empty($code['begins'])) {
      $code['begins'] = strtotime($code['begins']);
      if ($code['begins'] < 1) continue;
    }
    if (!empty($code['expires'])) {
      $code['expires'] = strtotime($code['expires']);
      if ($code['expires'] < 1) continue;
    }

    // Defaults
    if (empty($code['is_active'])) $code['is_active'] = 1;
    if (empty($code['maxuses']))   $code['maxuses'] = 1;
  
    $status = regcode_save_code($code, $action);
    if ($status) $success++;
  }
  
  // Status
  drupal_set_message(t('Successfully inserted @rows rows out of @total', array('@rows' => $success, '@total' => $rows)));
  
  // Close
  fclose($fh);
}



/**
 * Autocomplete helper for categories
 */
function regcode_admin_categories($string) {
  $matches = array();
  if (!empty($string)) {
    $result = db_query_range("SELECT DISTINCT category FROM {regcode} WHERE LOWER(category) LIKE LOWER('%s%%')", $string, 0, 10);
    while ($regcode = db_fetch_object($result)) {
      $matches[$regcode->category] = check_plain($regcode->category);
    }
  }
  
  print drupal_to_js($matches);
}


/**
 *
 */
function _regcode_admin_prepare_display($row) {
  $datefields = array('created', 'lastused', 'begins', 'expires');

  // Format the dates
  foreach ($datefields as $date) {
    if (empty($row[$date])) {
      $row[$date] = '-';
    } else {
      $row[$date] = sprintf('<span title="%s">%s</span>',
        format_date(strtotime($row[$date])),
        format_date(strtotime($row[$date]), 'custom', 'Y/n/j')
        );
    }
  }
  
  // Format the booleans
  $row['is_active'] = $row['is_active'] ? 'Yes' : 'No';
  
  // Format the UID
  if (!empty($row['uid'])) {
    $row['uid'] = l($row['uid'], 'user/' . $row['uid']);
  } else {
    $row['uid'] = '-';
  }
  
  return $row;
}